package pers_test

import (
	"testing"

	"git.sr.ht/~nelsam/hel/pkg/pers"
	"git.sr.ht/~nelsam/hel/vegr/ret"
	"github.com/poy/onpar"
	"github.com/poy/onpar/expect"
)

func TestPanic(t *testing.T) {
	o := onpar.BeforeEach(onpar.New(t), func(t *testing.T) expectation {
		return expect.New(t)
	})
	defer o.Run()

	o.Spec("it passes along a panic message and fills other fields with defaults", func(expect expectation) {
		type fooReturns struct {
			Foo ret.Panicker
			Bar chan bool
		}
		v := fooReturns{make(ret.Panicker, 1), make(chan bool, 1)}
		msg := "this is a panic message"
		pers.Panic(v, msg)
		expect(v.Foo).To(chain(receive(), equal(msg)))
		expect(v.Bar).To(chain(receive(), equal(false)))
	})

	o.Spec("it does not send on a blocker channel", func(expect expectation) {
		type fooReturns struct {
			Foo   ret.Panicker
			Block ret.Blocker
		}
		v := fooReturns{make(ret.Panicker, 1), make(ret.Blocker, 1)}
		msg := "this is a panic message"
		pers.Panic(v, msg)
		expect(v.Block).To(not(receive()))
	})
}
